import parseurl from 'parseurl'
import * as proxy from '../proxy.js'
import finalhandler from 'finalhandler'
import { HttpError } from 'http-errors'
import logger from '../logger.js'

export default function ({ urls, secure }) {
  return async (req, res, next) => {
    if (req.headers['proxied-by-ox-proxy']) return next()

    const { path } = parseurl(req)
    for (const [condition, { hostname, protocol, port }] of urls) {
      const isRegExp = condition instanceof RegExp
      const shouldHandle = isRegExp ? condition.test(path) : path.startsWith(condition)
      if (shouldHandle) {
        try {
          logger.info(`Proxy ${path} to ${protocol}//${hostname}:${port}`)
          await proxy.web(req, res, { hostname, protocol, port, secure })
          logger.debug(`Proxied ${path} to ${protocol}//${hostname}:${port}`)
          return
        } catch (err) {
          const handle = finalhandler(req, res)
          if (err instanceof HttpError) {
            logger.warn(`Could not proxy ${path}`)
            return handle(err)
          }
          logger.error(err)
          return handle()
        }
      }
    }

    next()
  }
}
